@@ -3,11 +3,12 @@ use strict;
 use warnings;
 use 5.010; # state
 # ABSTRACT: turn on Unicode - all of it
-our $VERSION = '0.010'; # VERSION
+our $VERSION = '0.011'; # VERSION
 
 
 use Import::Into;
 use parent qw(Encode charnames utf8 open warnings feature);
+use Symbol qw/qualify_to_ref/;
 
 sub import {
     my $target = caller;
@@ -38,15 +39,18 @@ sub _encode_argv {
 }
 
 sub _utf8_readdir(*) { ## no critic (Subroutines::ProhibitSubroutinePrototypes)
-    my $handle = shift;
+    my $pre_handle = shift;
+    my $handle = ref($pre_handle) ? $pre_handle : qualify_to_ref($pre_handle, caller);
+    my $hints = (caller 0)[10];
     if (wantarray) {
         my @all_files  = CORE::readdir($handle);
+        return @all_files if not $hints->{'utf8::all'};
         $_ = Encode::decode('UTF-8', $_) for @all_files;
         return @all_files;
     }
     else {
         my $next_file = CORE::readdir($handle);
-        $next_file = Encode::decode('UTF-8', $next_file);
+        $next_file = Encode::decode('UTF-8', $next_file) if $hints->{'utf8::all'};
         return $next_file;
     }
 }
@@ -66,7 +70,7 @@ utf8::all - turn on Unicode - all of it
 
 =head1 VERSION
 
-version 0.010
+version 0.011
 
 =head1 SYNOPSIS
 
